//
// Timer Eco
// AVR ATiny2313
// 2010.06.13
//
//
// Notice  Project-Configuration Option   Optimation -O1
//

#include <AVR/interrupt.h>
#include <avr/io.h>
#include <avr/wdt.h>
								//Timer mode
#define	MODE_NORMAL	0			//normal
#define	MODE_ECO	1			//eco

#define	TIMER_MAX	5			//Timer sw 0 - 5

								//LED control
#define	LED_OFF		0			//LED Off
#define	LED_ON		1			//LED On
#define	LED_SLOW	2			//LED blink of slow
#define	LED_NORMAL	3			//LED blink of noraml
#define	LED_HI		4			//LED blink of hi

								//Port A
#define	AC			0			//W: AC On/Off
#define	PORTA_1		1			//R: 

								//Port B
#define	LED_T1		0			//W: LED Time 1 (2h)
#define	LED_T2		1			//W: LED Time 2 (4h)
#define	LED_T3		2			//W: LED Time 3 (6h)
#define	LED_T4		3			//W: LED Time 4 (8h)
#define	LED_T5		4			//W: LED Time 5 (10h)

								//Port D
#define	TIME_SW		2			//R: Time SW
#define	MODE_SW		3			//R: Mode SW
#define	PORTD_4		4			//R: 
#define PORTD_5		5			//R:
#define PORTD_6		6			//R:

volatile static uint32_t	ucnt   = 0;				//Time Stik counter
volatile static uint8_t		adjst1 = 0;				//Time Adjstment counter 1
volatile static uint8_t		adjst2 = 0;				//Time Adjstment counter 2
static uint8_t	led_on[6] =  {0b00000000, 0b00000001, 0b00000011,
							  0b00000111, 0b00001111, 0b00011111};
static uint8_t	led_off[6] = {0b11100000, 0b11111110, 0b11111101,
							  0b11111011, 0b11110111, 0b11101111};

static uint8_t	led_no  = 0;						//LED No 0:Off 1-5:LED No
static uint8_t	led_sts = 0;						//LED 1:On 0:Off
static uint8_t	led_current_no = 0;					//LED Current no.

static uint8_t	led_mode = 0;	//LED Mode
static uint8_t	led_cnt  = 0;	//LED Blink counter


													//LED control
void Led_On ( void )								//LED On
{
	PORTB = PORTB | led_on[led_current_no];
	led_sts = LED_ON;
}

void Led_Off ( void )								//LED Off
{
	PORTB = PORTB & led_off[led_current_no];
	led_sts = LED_OFF;
}

void Led_Blink ( void )								//LED Blink
{
	if (led_sts != LED_OFF ) Led_Off();
	else                     Led_On();
}

void LedNoSet ( uint8_t no )						//Set of LED No.
{
	led_no = no;
}

void LedModeSet( uint8_t mode )						//LED Set of mode
{
	led_mode = mode;
}

void LedControl(void)								//LED control (CTC Interrup)
{
	if (led_current_no != led_no) {
		Led_Off();
		led_current_no = led_no;
	}
	led_cnt++;
	switch(	led_mode ) {							//Interval time 0.1mSec
	case LED_OFF:
		Led_Off();
		break;
	case LED_ON:
		Led_On();
		break;
	case LED_SLOW:
		if ( led_cnt > 13 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_NORMAL:
		if ( led_cnt > 5 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	case LED_HI:
		if ( led_cnt > 3 ) {
			Led_Blink();
			led_cnt = 0;
		}
		break;
	}
}

													//Timer 0: Time control
ISR ( TIMER0_COMPA_vect )							//Interrupt Timer 0 CTC mode Clock 1MHz
{
	ucnt++;											//0.1024 Sec
	if ( adjst1++ >= 50 ) {							//Adjst of 0.02Sec
		adjst1 = 0;
		ucnt++;
		if (adjst2++ >=250 ) {						//Adjst of 0.004Sec
			ucnt++;
			adjst2 = 0;
		}
	}
	LedControl();
}

void TimeInit( void )								//Initalize of Timer 0
{
	TCCR0B = 0b00000000;							//Stop to Timer 
	TCCR0A = 0b00000010;							//mode is CTC
	OCR0A  = 100;
	TCCR0B = 0b00000101;							//clock 1/1024 (Clock 1MHz)

	TIMSK  = _BV(OCIE0A);							//Interrupt of comp mach A

	ucnt   = 0;										//stick counter
	adjst1 = 0;										//adjstment counter 1
	adjst2 = 0;										//adjstment counter 2
}

uint8_t TimeCheck( uint32_t start, uint32_t time )	//Timer check
{
	uint8_t sts =0;

	if (ucnt >= start) {
		if (time >= start) {
			if (ucnt >= time) sts = 1;
		}
	}
	else {
		if (time >= start)    sts = 1;
		else {
			if (ucnt >= time) sts = 1;
		}
	} 
	return sts;
}

void TimeSet( uint32_t *start, uint32_t *time )		//Timer set
{
	*start = ucnt;
	*time  = *start + *time;
}

													//AC control
void AC_On ( void )									//AC power on
{
	PORTA |= _BV( AC );
}

void AC_Off ( void )								//AC power off
{
	PORTA &= ~_BV( AC );
}

													//Mode switch
uint8_t ModeSw()									//Read of mode switch
{													//1:eco  0:normal
	if ( PIND & 1<<MODE_SW ) return 0;				//Active low
	else                     return 1;
}

uint8_t	TimeSw()									//Time scale switch
{													//Read of Time off->2h->4h->6h->8h->10h
	if ( PIND & 1<<TIME_SW ) return 0;				//Active low
	else					 return 1;
}

void ChangeLED( uint8_t time_no, uint8_t on ) {
	LedNoSet( time_no );
	LedModeSet( LED_ON );
	if (on != 1) LedModeSet( LED_NORMAL );
	else         LedModeSet( LED_ON );
}


int main( void )
{
	uint32_t start, time;
	uint16_t timer = 0;
	uint8_t	mode, time_no, on, sw, sw2, init, no;

													//Port Initialize
	DDRA  = 0b00000001;								//0:AC
	DDRB  = 0b00011111;								//0-4:LED 1-5
	DDRD  = 0b00000000;								//2:Time SW, 3:Mode SW
	PORTA = 0b00000000;
	PORTB = 0b00000000;								//Pull Up
	PORTD = 0b00001100;								//Pull up 2:Time sw, 3:Mode sw
	TimeInit();

	sei();
	wdt_enable(WDTO_500MS);							//Enable of Watchdog Timer 500mSec

	on       = 0;
	start    = 0;
	time     = 0;
	mode = ModeSw();								//Mode 1:eco  0:normal

	time_no = 0;									//Defualt Timer 0 (Off)
	LedNoSet( time_no );
	LedModeSet( LED_OFF );
	AC_Off();
	
	init = 1;
	sw   = TimeSw();
	sw2  = 0;
	while (1) {
		if ( sw2 == 0 ) {
			if ( sw == 1 || init == 1) {
				if ( TimeSw() || init == 1) {
					if ( init == 1 ) {
						init = 0;
					}
					else {
						time_no ++;
						if ( time_no > TIMER_MAX ) time_no = 0;
						sw2 = 1;
					}

					start = time = 0;
					on    = 0;
					AC_Off();
					LedNoSet( time_no );
					switch( time_no) {
					case 0:										//Off
						LedModeSet( LED_OFF );
						timer = 0;
						break;
					case 1:										//2 hour
						LedModeSet( LED_ON );
						timer = 120;
						break;
					case 2:										//4 hour
						LedModeSet( LED_ON );
						timer = 240;
						break;
					case 3:										//6 hour
						LedModeSet( LED_ON );
						timer = 360;
						break;
					case 4:										//8 hour
						LedModeSet( LED_ON );
						timer = 480;
						break;
					case 5:										//10 hour
						LedModeSet( LED_ON );
						timer = 600;
						break;
					}
					if ( time_no != 0 ) {
						time = 4800;							//time of on : 8 minitue
						on   = 1;
						AC_On();
						TimeSet( &start, &time );
						LedModeSet( LED_ON );
					}
				}
			}
			else sw = TimeSw();
		}
		else {
			if ( TimeSw() == 0 ) sw = sw2 = 0;
		}
		if (timer > 0) {
			if (TimeCheck( start, time)) {
				if ( on ) {
					time = 4200;								//time of Off : 7 minute
					on = 0;
					if ( mode != MODE_NORMAL ) {
						LedModeSet( LED_NORMAL );
						AC_Off();
					}
					timer -= 8;
				}
				else {
					time = 4800;								//time of on : 8 minnute
					on = 1;
					LedModeSet( LED_ON );
					AC_On();
					timer -= 7;
				}
				TimeSet( &start, &time );
			}
			
			no = (timer + 59) / 60;
			switch( no ) {
			case 2:
				if (time_no != 1) {
					time_no = 1;
					ChangeLED( time_no, on );
				}
				break;
			case 4:
				if (time_no != 2) {
					time_no = 2;
					ChangeLED( time_no, on );
				}
				break;
			case 6:
				if (time_no != 3) {
					time_no = 3;
					ChangeLED( time_no, on );
				}
				break;
			case 8:
				if (time_no != 4) {
					time_no = 4;
					ChangeLED( time_no, on );
				}
				break;
			case 10:
				if (time_no != 5) {
					time_no = 5;
					ChangeLED( time_no, on );
				}
				break;
			}
		}
		else {
			if (on == 1) {
				on = 0;
				AC_Off();
				time_no = 0;
				LedModeSet( LED_OFF );
				LedNoSet( time_no );
			}
		}
		wdt_reset();											//Reset of Watchdog Timer
	}
}
